local rand = require 'random'
module(..., package.seeall)

local Timer = {}
Timer.functions = {}
function Timer.update(dt)
	local to_remove = {}
	for func, delay in pairs(Timer.functions) do
		delay = delay - dt
		if delay <= 0 then
			to_remove[#to_remove+1] = func
			func(func)
		else
			Timer.functions[func] = delay
		end
	end
	for _,func in ipairs(to_remove) do
		Timer.functions[func] = nil
	end
end

function Timer.add(delay, func)
	Timer.functions[func] = delay
end

function Interpolator(length, func)
	local t = 0
	return function(dt, ...)
		t = t + dt
		return t <= length or nil, func(math.min(1, (t-dt)/length), ...)
	end
end

local explosions = {}
local sprites = {}
local fader
function init(loader)
	sprites = loader.sprites
	fader = Interpolator(1.5, function(l)
		local l = math.sin(math.pi/2*l)^4
		local sw,sh = love.graphics.getWidth(), love.graphics.getHeight()
		loader.draw()
		love.graphics.setColor(0,0,0,l*255)
		love.graphics.rectangle('fill',0,0,sw,sh)
	end)

	for i = 1,25 do
		Timer.add(rand.uniform(.5,2.5), function()
			newExplosion(rand.uniform(.2,.8) * love.graphics.getWidth(), rand.uniform(.2,.8) * love.graphics.getHeight())
		end)
	end

	love.graphics.setFont(12)
end

function draw()
	-- draw fader
	fader(0)
	for _,explosion in pairs(explosions) do
		local progress = explosion.time -- 1s max
		local scale = math.tanh(progress*6)
		love.graphics.setColor(255,255,255,(1-progress)*255)
		for _,fire in ipairs(explosion.fireballs) do
			love.graphics.draw(fire.img, fire.x, fire.y, progress * fire.rot, scale,scale, sprite_size/2,sprite_size/2)
		end
	end
end

function update(dt)
	Timer.update(dt)
	for _,explosion in pairs(explosions) do
		for _,fire in ipairs(explosion.fireballs) do
			fire.x = fire.x + fire.dx * dt
			fire.y = fire.y + fire.dy * dt
		end
		explosion.time = explosion.time + dt
		if explosion.time > 1 then
			explosions[explosion] = nil
		end
	end
	fader(dt)
end

function asplode(x,y, dont_spawn_new)
	local fireballs = {}
	for i = 1,rand.int(10,15) do
		local angle = rand.uniform(math.pi, 2*math.pi)
		local speed = rand.int(50,100)

		fireballs[#fireballs+1] = {img = sprites.fire[ rand.int(1,#sprites.fire) ],
			x = x + rand.gauss(1)*20, y = y + rand.gauss(1)*20,
			dx = math.cos(angle) * speed, dy = math.sin(angle) * speed,
			rot = rand.uniform()*math.pi
		}
	end

	local explosion = {time = 0, fireballs = fireballs}
	explosions[explosion] = explosion

	if dont_spawn_new then return end

	Timer.add(rand.uniform(.2,.3), function()
		asplode(x,y, rand.uniform(0,1) < .7)
	end)
end
